  
 MODULE mod_chem_spack_rates
  
   IMPLICIT NONE
   PRIVATE
   PUBLIC :: rates ! subroutine
 CONTAINS
  
   SUBROUTINE rates(rk,y,w,ngas,ijkbeg,ijkend,maxblock_size,nr)
 
!------------------------------------------------------------------------
!
!     -- DESCRIPTION
!
!     This routine computes the reaction rates.
!     This routine is automatically generated by SPACK.
!     Mechanism: ../Mechanism/RACM   
!     Species: ../Mechanism/ciRA77 
!
!------------------------------------------------------------------------
!
!     -- INPUT VARIABLES
!
!     RK: kinetic rates.
!     Y: chemical concentrations.
!
!     -- INPUT/OUTPUT VARIABLES
!
!     -- OUTPUT VARIABLES
!
!     W: reaction rates.
!
!------------------------------------------------------------------------
!
!     -- REMARKS
!
!------------------------------------------------------------------------
!
!     -- MODIFICATIONS
!
!------------------------------------------------------------------------
!
!     -- AUTHOR(S)
!
!     SPACK.
!
!------------------------------------------------------------------------
 
      IMPLICIT NONE
 
 
 
      INTEGER  	, INTENT(IN)  :: ngas	               
      INTEGER  	, INTENT(IN)  :: ijkbeg 	       
      INTEGER  	, INTENT(IN)  :: ijkend 	       
      INTEGER  	, INTENT(IN)  :: maxblock_size	       
      INTEGER  	, INTENT(IN)  :: nr		       
      DOUBLE PRECISION , INTENT(IN)  :: rk(maxblock_size,nr)  
      DOUBLE PRECISION , INTENT(IN)  :: y(maxblock_size,NGAS) 
      DOUBLE PRECISION , INTENT(OUT) :: w(maxblock_size,nr)   
      INTEGER  	              :: ijk		       
 
      DO ijk=ijkbeg,ijkend
      w(ijk,  1) =  rk(ijk,  1) * Y(ijk,  4)
      w(ijk,  2) =  rk(ijk,  2) * Y(ijk,  1)
      w(ijk,  3) =  rk(ijk,  3) * Y(ijk,  1)
      w(ijk,  4) =  rk(ijk,  4) * Y(ijk,  7)
      w(ijk,  5) =  rk(ijk,  5) * Y(ijk,  8)
      w(ijk,  6) =  rk(ijk,  6) * Y(ijk,  9)
      w(ijk,  7) =  rk(ijk,  7) * Y(ijk,  5)
      w(ijk,  8) =  rk(ijk,  8) * Y(ijk,  5)
      w(ijk,  9) =  rk(ijk,  9) * Y(ijk,  2)
      w(ijk, 10) =  rk(ijk, 10) * Y(ijk, 32)
      w(ijk, 11) =  rk(ijk, 11) * Y(ijk, 32)
      w(ijk, 12) =  rk(ijk, 12) * Y(ijk, 33)
      w(ijk, 13) =  rk(ijk, 13) * Y(ijk, 44)
      w(ijk, 14) =  rk(ijk, 14) * Y(ijk, 45)
      w(ijk, 15) =  rk(ijk, 15) * Y(ijk, 46)
      w(ijk, 16) =  rk(ijk, 16) * Y(ijk, 34)
      w(ijk, 17) =  rk(ijk, 17) * Y(ijk, 35)
      w(ijk, 18) =  rk(ijk, 18) * Y(ijk, 35)
      w(ijk, 19) =  rk(ijk, 19) * Y(ijk, 36)
      w(ijk, 20) =  rk(ijk, 20) * Y(ijk, 37)
      w(ijk, 21) =  rk(ijk, 21) * Y(ijk, 41)
      w(ijk, 22) =  rk(ijk, 22) * Y(ijk, 38)
      w(ijk, 23) =  rk(ijk, 23) * Y(ijk, 40)
      w(ijk, 24) =  rk(ijk, 24) * Y(ijk, 13)
      w(ijk, 25) =  rk(ijk, 25) * Y(ijk, 13) * Y(ijk,  1)
      w(ijk, 26) =  rk(ijk, 26) * Y(ijk, 14)
      w(ijk, 27) =  rk(ijk, 27) * Y(ijk, 14)
      w(ijk, 28) =  rk(ijk, 28) * Y(ijk, 14)
      w(ijk, 29) =  rk(ijk, 29) * Y(ijk,  1) * Y(ijk, 15)
      w(ijk, 30) =  rk(ijk, 30) * Y(ijk,  1) * Y(ijk, 16)
      w(ijk, 31) =  rk(ijk, 31) * Y(ijk, 15) * Y(ijk, 16)
      w(ijk, 32) =  rk(ijk, 32) * Y(ijk,  2) * Y(ijk, 15)
      w(ijk, 33) =  rk(ijk, 33) * Y(ijk, 16) * Y(ijk, 16)
      w(ijk, 34) =  rk(ijk, 34) * Y(ijk, 16) * Y(ijk, 16)
      w(ijk, 35) =  rk(ijk, 35) * Y(ijk, 13) * Y(ijk,  3)
      w(ijk, 36) =  rk(ijk, 36) * Y(ijk, 13) * Y(ijk,  4)
      w(ijk, 37) =  rk(ijk, 37) * Y(ijk, 13) * Y(ijk,  4)
      w(ijk, 38) =  rk(ijk, 38) * Y(ijk, 15) * Y(ijk,  3)
      w(ijk, 39) =  rk(ijk, 39) * Y(ijk, 15) * Y(ijk,  4)
      w(ijk, 40) =  rk(ijk, 40) * Y(ijk, 15) * Y(ijk,  5)
      w(ijk, 41) =  rk(ijk, 41) * Y(ijk, 16) * Y(ijk,  3)
      w(ijk, 42) =  rk(ijk, 42) * Y(ijk, 16) * Y(ijk,  4)
      w(ijk, 43) =  rk(ijk, 43) * Y(ijk,  9)
      w(ijk, 44) =  rk(ijk, 44) * Y(ijk, 16) * Y(ijk,  5)
      w(ijk, 45) =  rk(ijk, 45) * Y(ijk, 15) * Y(ijk,  7)
      w(ijk, 46) =  rk(ijk, 46) * Y(ijk, 15) * Y(ijk,  8)
      w(ijk, 47) =  rk(ijk, 47) * Y(ijk, 15) * Y(ijk,  9)
      w(ijk, 48) =  rk(ijk, 48) * Y(ijk,  1) * Y(ijk,  3)
      w(ijk, 49) =  rk(ijk, 49) * Y(ijk,  1) * Y(ijk,  4)
      w(ijk, 50) =  rk(ijk, 50) * Y(ijk,  3) * Y(ijk,  3)
      w(ijk, 51) =  rk(ijk, 51) * Y(ijk,  5) * Y(ijk,  3)
      w(ijk, 52) =  rk(ijk, 52) * Y(ijk,  5) * Y(ijk,  4)
      w(ijk, 53) =  rk(ijk, 53) * Y(ijk,  5) * Y(ijk,  4)
      w(ijk, 54) =  rk(ijk, 54) * Y(ijk,  6)
      w(ijk, 55) =  rk(ijk, 55) * Y(ijk,  5) * Y(ijk,  5)
      w(ijk, 56) =  rk(ijk, 56) * Y(ijk, 15)
      w(ijk, 57) =  rk(ijk, 57) * Y(ijk, 15) * Y(ijk, 10)
      w(ijk, 58) =  rk(ijk, 58) * Y(ijk, 12) * Y(ijk, 15)
      w(ijk, 59) =  rk(ijk, 59) * Y(ijk, 26) * Y(ijk, 13)
      w(ijk, 60) =  rk(ijk, 60) * Y(ijk, 38) * Y(ijk, 13)
      w(ijk, 61) =  rk(ijk, 61) * Y(ijk, 17) * Y(ijk, 15)
      w(ijk, 62) =  rk(ijk, 62) * Y(ijk, 18) * Y(ijk, 15)
      w(ijk, 63) =  rk(ijk, 63) * Y(ijk, 19) * Y(ijk, 15)
      w(ijk, 64) =  rk(ijk, 64) * Y(ijk, 20) * Y(ijk, 15)
      w(ijk, 65) =  rk(ijk, 65) * Y(ijk, 21) * Y(ijk, 15)
      w(ijk, 66) =  rk(ijk, 66) * Y(ijk, 22) * Y(ijk, 15)
      w(ijk, 67) =  rk(ijk, 67) * Y(ijk, 23) * Y(ijk, 15)
      w(ijk, 68) =  rk(ijk, 68) * Y(ijk, 24) * Y(ijk, 15)
      w(ijk, 69) =  rk(ijk, 69) * Y(ijk, 25) * Y(ijk, 15)
      w(ijk, 70) =  rk(ijk, 70) * Y(ijk, 26) * Y(ijk, 15)
      w(ijk, 71) =  rk(ijk, 71) * Y(ijk, 27) * Y(ijk, 15)
      w(ijk, 72) =  rk(ijk, 72) * Y(ijk, 28) * Y(ijk, 15)
      w(ijk, 73) =  rk(ijk, 73) * Y(ijk, 29) * Y(ijk, 15)
      w(ijk, 74) =  rk(ijk, 74) * Y(ijk, 30) * Y(ijk, 15)
      w(ijk, 75) =  rk(ijk, 75) * Y(ijk, 31) * Y(ijk, 15)
      w(ijk, 76) =  rk(ijk, 76) * Y(ijk, 32) * Y(ijk, 15)
      w(ijk, 77) =  rk(ijk, 77) * Y(ijk, 33) * Y(ijk, 15)
      w(ijk, 78) =  rk(ijk, 78) * Y(ijk, 34) * Y(ijk, 15)
      w(ijk, 79) =  rk(ijk, 79) * Y(ijk, 40) * Y(ijk, 15)
      w(ijk, 80) =  rk(ijk, 80) * Y(ijk, 35) * Y(ijk, 15)
      w(ijk, 81) =  rk(ijk, 81) * Y(ijk, 36) * Y(ijk, 15)
      w(ijk, 82) =  rk(ijk, 82) * Y(ijk, 38) * Y(ijk, 15)
      w(ijk, 83) =  rk(ijk, 83) * Y(ijk, 15) * Y(ijk, 37)
      w(ijk, 84) =  rk(ijk, 84) * Y(ijk, 15) * Y(ijk, 39)
      w(ijk, 85) =  rk(ijk, 85) * Y(ijk, 44) * Y(ijk, 15)
      w(ijk, 86) =  rk(ijk, 86) * Y(ijk, 45) * Y(ijk, 15)
      w(ijk, 87) =  rk(ijk, 87) * Y(ijk, 46) * Y(ijk, 15)
      w(ijk, 88) =  rk(ijk, 88) * Y(ijk, 42) * Y(ijk, 15)
      w(ijk, 89) =  rk(ijk, 89) * Y(ijk, 43) * Y(ijk, 15)
      w(ijk, 90) =  rk(ijk, 90) * Y(ijk, 41) * Y(ijk, 15)
      w(ijk, 91) =  rk(ijk, 91) * Y(ijk, 32) * Y(ijk,  5)
      w(ijk, 92) =  rk(ijk, 92) * Y(ijk, 33) * Y(ijk,  5)
      w(ijk, 93) =  rk(ijk, 93) * Y(ijk, 35) * Y(ijk,  5)
      w(ijk, 94) =  rk(ijk, 94) * Y(ijk, 36) * Y(ijk,  5)
      w(ijk, 95) =  rk(ijk, 95) * Y(ijk, 38) * Y(ijk,  5)
      w(ijk, 96) =  rk(ijk, 96) * Y(ijk, 37) * Y(ijk,  5)
      w(ijk, 97) =  rk(ijk, 97) * Y(ijk, 31) * Y(ijk,  5)
      w(ijk, 98) =  rk(ijk, 98) * Y(ijk, 22) * Y(ijk,  5)
      w(ijk, 99) =  rk(ijk, 99) * Y(ijk, 23) * Y(ijk,  5)
      w(ijk,100) =  rk(ijk,100) * Y(ijk, 24) * Y(ijk,  5)
      w(ijk,101) =  rk(ijk,101) * Y(ijk, 25) * Y(ijk,  5)
      w(ijk,102) =  rk(ijk,102) * Y(ijk, 26) * Y(ijk,  5)
      w(ijk,103) =  rk(ijk,103) * Y(ijk, 27) * Y(ijk,  5)
      w(ijk,104) =  rk(ijk,104) * Y(ijk, 28) * Y(ijk,  5)
      w(ijk,105) =  rk(ijk,105) * Y(ijk, 43) * Y(ijk,  5)
      w(ijk,106) =  rk(ijk,106) * Y(ijk, 22) * Y(ijk,  1)
      w(ijk,107) =  rk(ijk,107) * Y(ijk, 23) * Y(ijk,  1)
      w(ijk,108) =  rk(ijk,108) * Y(ijk, 24) * Y(ijk,  1)
      w(ijk,109) =  rk(ijk,109) * Y(ijk, 25) * Y(ijk,  1)
      w(ijk,110) =  rk(ijk,110) * Y(ijk, 26) * Y(ijk,  1)
      w(ijk,111) =  rk(ijk,111) * Y(ijk, 27) * Y(ijk,  1)
      w(ijk,112) =  rk(ijk,112) * Y(ijk, 28) * Y(ijk,  1)
      w(ijk,113) =  rk(ijk,113) * Y(ijk, 38) * Y(ijk,  1)
      w(ijk,114) =  rk(ijk,114) * Y(ijk, 37) * Y(ijk,  1)
      w(ijk,115) =  rk(ijk,115) * Y(ijk, 43) * Y(ijk,  1)
      w(ijk,116) =  rk(ijk,116) * Y(ijk, 60) * Y(ijk,  4)
      w(ijk,117) =  rk(ijk,117) * Y(ijk, 60) * Y(ijk, 16)
      w(ijk,118) =  rk(ijk,118) * Y(ijk, 61) * Y(ijk,  4)
      w(ijk,119) =  rk(ijk,119) * Y(ijk, 61)
      w(ijk,120) =  rk(ijk,120) * Y(ijk, 61) * Y(ijk,  1)
      w(ijk,121) =  rk(ijk,121) * Y(ijk, 62) * Y(ijk,  4)
      w(ijk,122) =  rk(ijk,122) * Y(ijk, 62)
      w(ijk,123) =  rk(ijk,123) * Y(ijk, 62) * Y(ijk,  1)
      w(ijk,124) =  rk(ijk,124) * Y(ijk, 63) * Y(ijk,  4)
      w(ijk,125) =  rk(ijk,125) * Y(ijk, 63)
      w(ijk,126) =  rk(ijk,126) * Y(ijk, 63) * Y(ijk,  1)
      w(ijk,127) =  rk(ijk,127) * Y(ijk, 67) * Y(ijk,  4)
      w(ijk,128) =  rk(ijk,128) * Y(ijk, 42)
      w(ijk,129) =  rk(ijk,129) * Y(ijk, 68) * Y(ijk,  4)
      w(ijk,130) =  rk(ijk,130) * Y(ijk, 43)
      w(ijk,131) =  rk(ijk,131) * Y(ijk, 49) * Y(ijk,  3)
      w(ijk,132) =  rk(ijk,132) * Y(ijk, 50) * Y(ijk,  3)
      w(ijk,133) =  rk(ijk,133) * Y(ijk, 51) * Y(ijk,  3)
      w(ijk,134) =  rk(ijk,134) * Y(ijk, 52) * Y(ijk,  3)
      w(ijk,135) =  rk(ijk,135) * Y(ijk, 53) * Y(ijk,  3)
      w(ijk,136) =  rk(ijk,136) * Y(ijk, 54) * Y(ijk,  3)
      w(ijk,137) =  rk(ijk,137) * Y(ijk, 55) * Y(ijk,  3)
      w(ijk,138) =  rk(ijk,138) * Y(ijk, 56) * Y(ijk,  3)
      w(ijk,139) =  rk(ijk,139) * Y(ijk, 57) * Y(ijk,  3)
      w(ijk,140) =  rk(ijk,140) * Y(ijk, 58) * Y(ijk,  3)
      w(ijk,141) =  rk(ijk,141) * Y(ijk, 59) * Y(ijk,  3)
      w(ijk,142) =  rk(ijk,142) * Y(ijk, 64) * Y(ijk,  3)
      w(ijk,143) =  rk(ijk,143) * Y(ijk, 65) * Y(ijk,  3)
      w(ijk,144) =  rk(ijk,144) * Y(ijk, 66) * Y(ijk,  3)
      w(ijk,145) =  rk(ijk,145) * Y(ijk, 67) * Y(ijk,  3)
      w(ijk,146) =  rk(ijk,146) * Y(ijk, 68) * Y(ijk,  3)
      w(ijk,147) =  rk(ijk,147) * Y(ijk, 69) * Y(ijk,  3)
      w(ijk,148) =  rk(ijk,148) * Y(ijk, 70) * Y(ijk,  3)
      w(ijk,149) =  rk(ijk,149) * Y(ijk, 71) * Y(ijk,  3)
      w(ijk,150) =  rk(ijk,150) * Y(ijk, 49) * Y(ijk, 16)
      w(ijk,151) =  rk(ijk,151) * Y(ijk, 50) * Y(ijk, 16)
      w(ijk,152) =  rk(ijk,152) * Y(ijk, 51) * Y(ijk, 16)
      w(ijk,153) =  rk(ijk,153) * Y(ijk, 52) * Y(ijk, 16)
      w(ijk,154) =  rk(ijk,154) * Y(ijk, 53) * Y(ijk, 16)
      w(ijk,155) =  rk(ijk,155) * Y(ijk, 54) * Y(ijk, 16)
      w(ijk,156) =  rk(ijk,156) * Y(ijk, 55) * Y(ijk, 16)
      w(ijk,157) =  rk(ijk,157) * Y(ijk, 56) * Y(ijk, 16)
      w(ijk,158) =  rk(ijk,158) * Y(ijk, 57) * Y(ijk, 16)
      w(ijk,159) =  rk(ijk,159) * Y(ijk, 58) * Y(ijk, 16)
      w(ijk,160) =  rk(ijk,160) * Y(ijk, 59) * Y(ijk, 16)
      w(ijk,161) =  rk(ijk,161) * Y(ijk, 64) * Y(ijk, 16)
      w(ijk,162) =  rk(ijk,162) * Y(ijk, 65) * Y(ijk, 16)
      w(ijk,163) =  rk(ijk,163) * Y(ijk, 66) * Y(ijk, 16)
      w(ijk,164) =  rk(ijk,164) * Y(ijk, 67) * Y(ijk, 16)
      w(ijk,165) =  rk(ijk,165) * Y(ijk, 67) * Y(ijk, 16)
      w(ijk,166) =  rk(ijk,166) * Y(ijk, 68) * Y(ijk, 16)
      w(ijk,167) =  rk(ijk,167) * Y(ijk, 68) * Y(ijk, 16)
      w(ijk,168) =  rk(ijk,168) * Y(ijk, 69) * Y(ijk, 16)
      w(ijk,169) =  rk(ijk,169) * Y(ijk, 70) * Y(ijk, 16)
      w(ijk,170) =  rk(ijk,170) * Y(ijk, 71) * Y(ijk, 16)
      w(ijk,171) =  rk(ijk,171) * Y(ijk, 49) * Y(ijk, 49)
      w(ijk,172) =  rk(ijk,172) * Y(ijk, 50) * Y(ijk, 49)
      w(ijk,173) =  rk(ijk,173) * Y(ijk, 51) * Y(ijk, 49)
      w(ijk,174) =  rk(ijk,174) * Y(ijk, 52) * Y(ijk, 49)
      w(ijk,175) =  rk(ijk,175) * Y(ijk, 53) * Y(ijk, 49)
      w(ijk,176) =  rk(ijk,176) * Y(ijk, 54) * Y(ijk, 49)
      w(ijk,177) =  rk(ijk,177) * Y(ijk, 55) * Y(ijk, 49)
      w(ijk,178) =  rk(ijk,178) * Y(ijk, 56) * Y(ijk, 49)
      w(ijk,179) =  rk(ijk,179) * Y(ijk, 57) * Y(ijk, 49)
      w(ijk,180) =  rk(ijk,180) * Y(ijk, 58) * Y(ijk, 49)
      w(ijk,181) =  rk(ijk,181) * Y(ijk, 59) * Y(ijk, 49)
      w(ijk,182) =  rk(ijk,182) * Y(ijk, 64) * Y(ijk, 49)
      w(ijk,183) =  rk(ijk,183) * Y(ijk, 65) * Y(ijk, 49)
      w(ijk,184) =  rk(ijk,184) * Y(ijk, 66) * Y(ijk, 49)
      w(ijk,185) =  rk(ijk,185) * Y(ijk, 67) * Y(ijk, 49)
      w(ijk,186) =  rk(ijk,186) * Y(ijk, 67) * Y(ijk, 49)
      w(ijk,187) =  rk(ijk,187) * Y(ijk, 68) * Y(ijk, 49)
      w(ijk,188) =  rk(ijk,188) * Y(ijk, 68) * Y(ijk, 49)
      w(ijk,189) =  rk(ijk,189) * Y(ijk, 69) * Y(ijk, 49)
      w(ijk,190) =  rk(ijk,190) * Y(ijk, 70) * Y(ijk, 49)
      w(ijk,191) =  rk(ijk,191) * Y(ijk, 71) * Y(ijk, 49)
      w(ijk,192) =  rk(ijk,192) * Y(ijk, 50) * Y(ijk, 67)
      w(ijk,193) =  rk(ijk,193) * Y(ijk, 51) * Y(ijk, 67)
      w(ijk,194) =  rk(ijk,194) * Y(ijk, 52) * Y(ijk, 67)
      w(ijk,195) =  rk(ijk,195) * Y(ijk, 53) * Y(ijk, 67)
      w(ijk,196) =  rk(ijk,196) * Y(ijk, 54) * Y(ijk, 67)
      w(ijk,197) =  rk(ijk,197) * Y(ijk, 55) * Y(ijk, 67)
      w(ijk,198) =  rk(ijk,198) * Y(ijk, 56) * Y(ijk, 67)
      w(ijk,199) =  rk(ijk,199) * Y(ijk, 57) * Y(ijk, 67)
      w(ijk,200) =  rk(ijk,200) * Y(ijk, 58) * Y(ijk, 67)
      w(ijk,201) =  rk(ijk,201) * Y(ijk, 59) * Y(ijk, 67)
      w(ijk,202) =  rk(ijk,202) * Y(ijk, 64) * Y(ijk, 67)
      w(ijk,203) =  rk(ijk,203) * Y(ijk, 65) * Y(ijk, 67)
      w(ijk,204) =  rk(ijk,204) * Y(ijk, 66) * Y(ijk, 67)
      w(ijk,205) =  rk(ijk,205) * Y(ijk, 67) * Y(ijk, 67)
      w(ijk,206) =  rk(ijk,206) * Y(ijk, 68) * Y(ijk, 67)
      w(ijk,207) =  rk(ijk,207) * Y(ijk, 69) * Y(ijk, 67)
      w(ijk,208) =  rk(ijk,208) * Y(ijk, 70) * Y(ijk, 67)
      w(ijk,209) =  rk(ijk,209) * Y(ijk, 71) * Y(ijk, 67)
      w(ijk,210) =  rk(ijk,210) * Y(ijk, 70) * Y(ijk, 70)
      w(ijk,211) =  rk(ijk,211) * Y(ijk, 70) * Y(ijk, 71)
      w(ijk,212) =  rk(ijk,212) * Y(ijk, 71) * Y(ijk, 71)
      w(ijk,213) =  rk(ijk,213) * Y(ijk, 49) * Y(ijk,  5)
      w(ijk,214) =  rk(ijk,214) * Y(ijk, 50) * Y(ijk,  5)
      w(ijk,215) =  rk(ijk,215) * Y(ijk, 51) * Y(ijk,  5)
      w(ijk,216) =  rk(ijk,216) * Y(ijk, 52) * Y(ijk,  5)
      w(ijk,217) =  rk(ijk,217) * Y(ijk, 53) * Y(ijk,  5)
      w(ijk,218) =  rk(ijk,218) * Y(ijk, 54) * Y(ijk,  5)
      w(ijk,219) =  rk(ijk,219) * Y(ijk, 55) * Y(ijk,  5)
      w(ijk,220) =  rk(ijk,220) * Y(ijk, 56) * Y(ijk,  5)
      w(ijk,221) =  rk(ijk,221) * Y(ijk, 57) * Y(ijk,  5)
      w(ijk,222) =  rk(ijk,222) * Y(ijk, 58) * Y(ijk,  5)
      w(ijk,223) =  rk(ijk,223) * Y(ijk, 59) * Y(ijk,  5)
      w(ijk,224) =  rk(ijk,224) * Y(ijk, 64) * Y(ijk,  5)
      w(ijk,225) =  rk(ijk,225) * Y(ijk, 65) * Y(ijk,  5)
      w(ijk,226) =  rk(ijk,226) * Y(ijk, 66) * Y(ijk,  5)
      w(ijk,227) =  rk(ijk,227) * Y(ijk, 67) * Y(ijk,  5)
      w(ijk,228) =  rk(ijk,228) * Y(ijk, 68) * Y(ijk,  5)
      w(ijk,229) =  rk(ijk,229) * Y(ijk, 69) * Y(ijk,  5)
      w(ijk,230) =  rk(ijk,230) * Y(ijk, 70) * Y(ijk,  5)
      w(ijk,231) =  rk(ijk,231) * Y(ijk, 71) * Y(ijk,  5)
      w(ijk,232) =  rk(ijk,232) * Y(ijk, 72) * Y(ijk, 16)
      w(ijk,233) =  rk(ijk,233) * Y(ijk, 72) * Y(ijk, 49)
      w(ijk,234) =  rk(ijk,234) * Y(ijk, 72) * Y(ijk, 67)
      w(ijk,235) =  rk(ijk,235) * Y(ijk, 72) * Y(ijk, 72)
      w(ijk,236) =  rk(ijk,236) * Y(ijk, 72) * Y(ijk,  3)
      w(ijk,237) =  rk(ijk,237) * Y(ijk, 72) * Y(ijk,  5)
       END DO
 
   END SUBROUTINE rates
 
  END MODULE mod_chem_spack_rates
 
