!========================================================================
! Soil Moisture Estimate for NWP Models
! Coded and implemented by Rodrigo Gevaerd and Saulo Freitas
! Ref.: Gevaerd, R. e S. R. Freitas, Estimativa operacional da umidade
! do solo para iniciacao de modelos de previsao numerica da atmosfera.
! Parte I: Descricao da metodologia e validacao. Rev. Bras. Meteo.,
! volume especial do LBA, 2007.
!========================================================================
!!***DBG***
!iErrNumber=debug("POntos em Lon",'LFR',srcName,procedureName,c_continue,0,0,nlon)
!iErrNumber=debug("POntos em Lat",'LFR',srcName,procedureName,c_continue,0,0,nlat)
!iErrNumber=debug("POntos em Lev",'LFR',srcName,procedureName,c_continue,0,0,n4us)
!!***DBG***
MODULE soilMoisture

  use ModNamelistFile, only: namelistFile

  use memSoilMoisture, only: &
       soil_moist, soil_moist_fail, usdata_in, usmodel_in ! INTENT(IN)

  IMPLICIT NONE

  PUBLIC  :: soilMoistureInit
  PUBLIC  :: StoreNamelistFileAtSoilMoisture
!!$  PRIVATE :: gatherData
!!$  PRIVATE :: gatherData2D
!!$  PRIVATE :: gatherData4D
!  PUBLIC :: apiPrlatlon
!  PUBLIC :: interpolacao
!  PRIVATE :: changeDay

!!$  interface gatherData
!!$     module procedure gatherData2D, gatherData4D
!!$  end interface

contains

  subroutine soilmoistureinit(n1, n2, n3, mzg, mzs, npat, ifm, &
       theta, pi0, pp,                                           &
       soil_water, soil_energy, soil_text,                       &
       glat, glon,                                               &
       lpw_r                                                       &
       )
    !# _
	!#
	!# @note
	!#
	!#
	!# **Brief**: _
	!#
	!# **Documentation**: <LF>
	!#
	!# **Author**: Luiz Flavio **&#9993;**<mailto:lufla@enterprise>
	!#
	!# **Date**: 2020-04-30
	!# @endnote
	!#
	!# @changes
	!#**Changelogs:**
	!# +
	!# @endchanges
	!# @bug
	!# **Open bugs:**
	!#
	!# @endbug
	!#
	!# @todo
	!# **Todo list:**n	!#
	!# @endtodo
	!#
	!# @warning
	!# Now is under CC-GPL License, please see
	!# &copy; <https://creativecommons.org/licenses/GPL/2.0/legalcode.pt>
	!# @endwarning
	!#
	!#--- ----------------------------------------------------------------------------------------
	use dump
    use mem_grid, only : runtype,  &          ! intent(in)
         iyear1, imonth1, idate1, itime1, &   ! intent(in)
         nnxp, nnyp, nnzp,                &   ! intent(in)
         globalsizes                          ! subroutine

    use io_params, only : timstr     ! intent(in)

    use rconstants, only : cpi       ! intent(in)

    use leaf_coms, only : soilcp,  & ! intent(in)
         slmsts,                   & ! intent(in)
         slcpd                       ! intent(in)

    use mem_leaf, only : stgoff,   & ! intent(in)
         slmstr,                   & ! intent(in)
         slz                         ! intent(in)

    use node_mod, only: &
         nodei0, nodej0, & ! intent(in)
         nodemxp, nodemyp, & ! intent(in)
         nmachs,  & ! intent(in)
         mynum,  &  ! intent(in)
         mchnum, &  ! intent(in)
         master_num ! intent(in)

    use parlib, only: &
         parf_bcast ! subroutine

    use mem_aerad, only: &
         nwave ! intent(in)

    use readbcst, only: &
         gatherdata
	implicit none

	character(len=*),parameter :: procedureName="soilmoistureinit"
	character(len=*),parameter :: srcName="soilMoisture.f90"
	include "constants.f90"
    include "i8.h"
    include "files.h"


    ! arguments:
    integer, intent(in) :: n1, n2, n3, mzg, mzs, npat, ifm
!!$    real, intent(in)    :: theta(n1,n2,n3)
!!$    real, intent(in)    :: pi0(n1,n2,n3)
!!$    real, intent(in)    :: pp(n1,n2,n3)
!!$    real, intent(in)    :: glat(n2,n3)
!!$    real, intent(in)    :: glon(n2,n3)
!!$    integer, intent(in) :: lpw(n2,n3)
!!$    real, intent(inout) :: soil_water(mzg,n2,n3,npat)
!!$    real, intent(inout) :: soil_energy(mzg,n2,n3,npat)
!!$    real, intent(in)    :: soil_text(mzg,n2,n3,npat)
    real, intent(in)    :: theta(:,:,:)         !(n1,n2,n3)
    real, intent(in)    :: pi0(:,:,:)           !(n1,n2,n3)
    real, intent(in)    :: pp(:,:,:)            !(n1,n2,n3)
    real, intent(in)    :: glat(:,:)            !(n2,n3)
    real, intent(in)    :: glon(:,:)            !(n2,n3)
    real, intent(in) :: lpw_r(:,:)             !(n2,n3)
    real, intent(inout) :: soil_water(:,:,:,:)  !(mzg,n2,n3,npat)
    real, intent(inout) :: soil_energy(:,:,:,:) !(mzg,n2,n3,npat)
    real, intent(in)    :: soil_text(:,:,:,:)   !(mzg,n2,n3,npat)

    ! local variables:

    integer :: lpw(n2,n3)             !(n2,n3)
    integer            :: i, j, k, ipat, nveg, nsoil
    real               :: c1, airtemp, pis
    real               :: globalsoilwater(mzg, nnxp(ifm), nnyp(ifm), npat)
    real               :: globalsoiltext(mzg, nnxp(ifm), nnyp(ifm), npat)
    real               :: globalglon(nnxp(ifm), nnyp(ifm))
    real               :: globalglat(nnxp(ifm), nnyp(ifm))
    integer            :: qi1, qi2, qj1, qj2, ncount,               &
         ii, jj, jc, ic, i1, j1, i2, j2, kk, ifname, k2, &
         ipref, ipref_start, icihourmin
    integer            :: n4us
    integer            :: nlat, nlon
    real, allocatable  :: slz_us(:)
    real               :: latni, latnf, lonni, lonnf, ilatn, ilonn, &
         ilats, ilons, latn, lonn, lats, lons, dlatr, dlonr
    logical            :: there, theref
    character(len=f_name_length) :: usdata, usmodel
    character(len=50)  :: pref
    character(len=2)   :: cidate, cimon
    character(len=1)   :: cgrid
    character(len=4)   :: ciyear
    character(len=4)   :: cihourmin
    real, allocatable  :: api_us(:,:,:), prlat(:,:), prlon(:,:), usdum(:),api_temp(:,:,:)
    real               :: dif_time, dummy
    integer            :: int_dif_time, idate2, imonth2, iyear2, hourmin, &
         da, sair
    integer            :: ierr
!!$    integer, parameter :: idim_type_min = 2
!!$    integer, parameter :: idim_type_max = 7
    integer, parameter :: idim_type     = 4
!!$    integer            :: il1(nmachs)
!!$    integer            :: ir2(nmachs)
!!$    integer            :: jb1(nmachs)
!!$    integer            :: jt2(nmachs)
!!$    integer            :: localsize(nmachs,idim_type_min:idim_type_max)
!!$    integer            :: disp(nmachs,idim_type_min:idim_type_max)
!!$    integer            :: maxlocalsize
!!$    integer            :: sizegathered(idim_type_min:idim_type_max)
!!$    integer            :: maxsizegathered
!!$    integer            :: sizefullfield(idim_type_min:idim_type_max)
!!$    integer            :: maxsizefullfield
!!$    integer            :: globalsize(idim_type_min:idim_type_max)
!!$    real, allocatable  :: localchunk(:)
!!$    real, allocatable  :: gathered(:)
!!$    real, allocatable  :: fullfield(:)
	real :: usdum_before(200,200,50)
    character(len=16)  :: varn
    integer            :: i0, j0, ia, iz, ja, jz,irec,nrec
    logical            :: general
    character(len=256) :: dset,title,tdef,lixo
    character(len=32),allocatable :: soilVarName(:)
    integer, allocatable :: slevels(:)
    real,allocatable :: us_geos(:,:,:)
    real,allocatable :: lats_(:),lons_(:)
    real :: undef
    integer :: vars,ki


    namelist /gradeumso/ latni, latnf, lonni, lonnf, ilatn, ilonn, nlat, nlon

    lpw=int(lpw_r)
    ! initial grid definitions for grid ifm
    i0 = nodei0(mynum,ifm)
    j0 = nodej0(mynum,ifm)
    ia = nodei0(mynum,ifm) + 1
    iz = nodei0(mynum,ifm) + nodemxp(mynum,ifm)
    ja = nodej0(mynum,ifm) + 1
    jz = nodej0(mynum,ifm) + nodemyp(mynum,ifm)

    iyear2  = iyear1
    imonth2 = imonth1
    idate2  = idate1

    ! determinacao do tipo de produto de umidade
    do i=256,1,-1
       if (usdata_in(i:i)=='/') then
          ipref_start = i + 1
          exit
       endif

       !   print*,'i=',i
       !   print*,usdata(i+1:ifname)
       !   stop
       !  endif
    enddo


    ! definicao da espessura das camadas
    ipref = len_trim(usdata_in)
    pref = usdata_in(ipref_start:ipref)

    if (pref=='SM_V2.') then
       n4us = 6                    ! modelo v2 com 6 camadas
       allocate(slz_us(0:n4us))
       slz_us = (/-3.0, -2.0, -1.0, -0.5, -0.25, -0.1, 0. /)
    elseif (pref=='GL_SM.GPCP.' .or.  pref=='GL_SM.GPNR.' ) then
       n4us = 8                    ! modelo glsm v2 com 8 camadas
       allocate(slz_us(0:n4us))
       slz_us = (/-4.5, -2.5, -1.75, -1.0, -0.5, -0.25, -0.13, -0.05, 0./)
    elseif (pref=='GFS025GR.SOILM.') then
       n4us = 4                    ! modelo do gfs 4 camadas
       allocate(slz_us(0:n4us))
       slz_us = (/-2.0, -1.0,  -0.40, -0.10, 0./)
    elseif (pref=='SM.GEOS.') then
	   !Do nothing
!	   n4us = 8                    ! modelo geos5 com 8 camadas
!       allocate(slz_us(0:n4us))
!       slz_us = (/-3.25,-2.125,-1.375,-0.75,-0.375,-0.19,-0.09,-0.025, 0./)
    else
       n4us = 4                    ! modelo original com 4 camadas
       allocate(slz_us(n4us))
       slz_us = (/-2.4, -0.4, -0.1, 0./)
    endif

    ! composicao do nome dos arquivos de entrada e saida

    if ( (runtype(1:7)=='history') .and.                 &
         ((soil_moist=='h') .or. (soil_moist=='h') .or.  &
         (soil_moist=='a') .or. (soil_moist=='a'))) then

       dif_time = timstr

       !if (timeunit == 'h') then
       !int_dif_time = floor(dif_time/24.)
       !elseif (timeunit == 'm') then
       !int_dif_time = floor(dif_time/1440.)
       !elseif (timeunit == 's') then
       int_dif_time = floor(dif_time/86400.)
       !endif

       call changeday(idate1, imonth1, iyear1, int_dif_time,   &
            idate2, imonth2, iyear2)

    else

       int_dif_time = 0

    endif

    if ((soil_moist_fail=='l') .or. (soil_moist_fail=='l')) then
       ! looking for until 5 days old file
       da = 5
    else
       da = 1
    endif

    sair = 0

    do i=1,da

       write (cidate, '(i2.2)') idate2
       write (cimon,  '(i2.2)') imonth2
       write (ciyear, '(i4)')   iyear2
       write (cgrid,  '(i1)')   ifm

       ! calculating the hour of simulation
       if ((itime1>=0000) .and. (itime1<1200)) then
          hourmin = 0000
          if (pref=='GL_SM.GPCP.' .or. pref=='GL_SM.GPNR.' .or. pref=='GFS025GR.SOILM.') hourmin = 00
       else
          hourmin = 1200
          if (pref=='GL_SM.GPCP.' .or. pref=='GL_SM.GPNR.' .or. pref=='GFS025GR.SOILM.') hourmin = 12
       endif

       if (pref=='GL_SM.GPCP.' .or. pref=='GL_SM.GPNR.' .or. pref=='GFS025GR.SOILM.') then
          write (cihourmin, '(i2.2)') hourmin
          icihourmin = 2
       else
          write (cihourmin, '(i4.4)') hourmin
          icihourmin = 4
       endif
       !ipref=index(usdata_in,' ')
       !pref=usdata_in(ipref-2:ipref-1)
       !ipref = len_trim(usdata_in)

       !     if (usdata_in(ipref_start:ipref) == 'us'.or.usdata_in(ipref_start:ipref) == 'sm') then
       !        pref = usdata_in(ipref_start:ipref)
       !     endif
       !     if (usdata_in(ipref_start:ipref) == 'sm_v2.' .or. usdata_in(ipref_start:ipref) == 'gl_sm.gpcp.') then
       !        pref = usdata_in(ipref_start:ipref)
       !     endif

       if (pref=='us') then
          cihourmin  = ''
          icihourmin = 0
       endif

       usdata = trim(usdata_in)//ciyear//cimon//cidate//cihourmin(1:icihourmin)//'.vfm'

       ifname = len_trim(usdata)

       if (mchnum==master_num) inquire(file=usdata(1:ifname),exist=theref)
       call parf_bcast(theref, master_num)

       if (.not.theref) &
            usdata = trim(usdata_in)//ciyear//cimon//cidate//cihourmin(1:icihourmin)//'.gra'

       usmodel = trim(usmodel_in)//ciyear//cimon//cidate//cihourmin(1:icihourmin)//'_g'//cgrid//'.mod'

       if (mchnum==master_num) then
          print *, 'looking up soil moisture date files: '
          print *, '  usdata : ', usdata(1:len_trim(usdata))
          print *, '  usmodel: ', usmodel(1:len_trim(usmodel))

          ifname = len_trim(usmodel)
          inquire(file=usmodel(1:ifname), exist=there)

          if(.not.there) then

             ifname = len_trim(usdata)
             inquire(file=usdata(1:ifname), exist=there)

             if (there) then
                sair = 1
             else
                print *, 'files:', usdata(1:ifname), &
                     ' and: ', usmodel(1:ifname), ' not found!'
             endif
          else
             sair = 1
          endif
       endif !(mchnum==master_num)

       call parf_bcast(sair, master_num)

       if (sair==1) exit

       call changeday(idate1, imonth1, iyear1, (int_dif_time - i),   &
            idate2, imonth2, iyear2)

    enddo

    ifname = len_trim(usmodel)

    if (mchnum==master_num) inquire(file=usmodel(1:ifname), exist=there)
    call parf_bcast(there, master_num)


!- 8/9/2015 srf
!    because the the soil moisture dataset interpolated to the model grid
!    is never checked again, we will allways require the interpolation to
!    be done every model initialization. this will prevents soil moisture
!    interpolated for given grid specification being erroneously be used
!    for a different model configuration
!
!    if (.not.there) then
!-srf end
       ifname = len_trim(usdata)
       if (mchnum==master_num) inquire(file=usdata(1:ifname), exist=there)
       call parf_bcast(there, master_num)

       if (.not.there) then
          if (mchnum==master_num) then
             print *, '  usdata : ', usdata(1:ifname)
             print *, '  usmodel: ', usmodel(1:len_trim(usmodel))
          end if
          if ((soil_moist_fail=='s') .or. (soil_moist_fail=='s')) then
             call fatal_error('* heterogenous soil moisture init. error! '//&
                  '  program will stop!')
             !!stop
          else
             if (mchnum==master_num) then
                print *, '  homogeneous soil moisture initialization.'
             end if
             return
          endif
       endif

       !srf-2015
       !if (mchnum==master_num) then
          !print*,'----------------------------------------------'
          !print*,'  homogenous soil moisture initialization in'
          !print*,'     point outside the south america'
          !print*,'----------------------------------------------'
       !end if

       c1 = 0.5*cpi

       do j=1,n3
          do i=1,n2

             k2  = lpw(i,j)
             pis = c1*(pi0(k2-1,i,j) + pi0(k2,i,j) + pp(k2-1,i,j) + pp(k2,i,j))
             airtemp = theta(k2,i,j)*pis

             do ipat=2,npat
                do k=1,mzg
                   nsoil = nint(soil_text(k,i,j,ipat))
                   soil_water(k,i,j,ipat) = &
                        max(soilcp(nsoil), slmstr(k)*slmsts(nsoil))
                   soil_energy(k,i,j,ipat) = &
                        (airtemp - 273.15 + stgoff(k))*  &
                        (slcpd(nsoil) + soil_water(k,i,j,ipat)*4.186e6) + &
                        soil_water(k,i,j,ipat)*3.34e8
                enddo
             enddo
          enddo
       enddo

       !--------------------------------------------------------- heterogeneous
       if (mchnum==master_num) then
          print*,'----------------------------------------------'
          print*,'       soil moisture initialization'
          print*,'----------------------------------------------'
       endif

       ! dados da grade de precipitacao

       if (mchnum==master_num) &
            inquire(file=trim(usdata_in)//'_ent', exist=general)

       call parf_bcast(general, master_num)

       if (general) then
          if (mchnum==master_num) then
             open (unit=93,file=usdata_in(1:len_trim(usdata_in))//'_ent',status='old')
             read (unit=93,nml=gradeumso) !latni, latnf, lonni, lonnf, ilatn, ilonn, nlat, nlon
             close (unit=93, status='keep')
          endif
          call parf_bcast(latni, master_num)
          call parf_bcast(latnf, master_num)
          call parf_bcast(lonni, master_num)
          call parf_bcast(lonnf, master_num)
          call parf_bcast(ilatn, master_num)
          call parf_bcast(ilonn, master_num)
          call parf_bcast(nlat,  master_num)
          call parf_bcast(nlon,  master_num)

       elseif (pref=='us') then
          latni =  -45.
          latnf =   12.9477
          lonni =  -82.
          lonnf =  -30.055
          ilatn =    0.0359477
          ilonn =    0.0382513
          nlat  = 1613
          nlon  = 1359
       elseif (pref=='SM') then
          latni =  -50.125
          latnf =   40.125
          lonni = -120.125
          lonnf =   60.125
          ilatn =    0.250
          ilonn =    0.250
          nlat  =  362
          nlon  =  722
       elseif (pref=='SM_V2.') then
          latni =  -59.875              !sm v2  trmm global
          latnf =   59.875
          lonni = -179.875
          lonnf =  179.875
          ilatn =    0.250
          ilonn =    0.250
          nlat  =  480
          nlon  = 1440
       elseif (pref=='GL_SM.GPCP.') then
          latni =  -89.5            !  gpcp global
          latnf =   89.5
          lonni = -179.5
          lonnf =  179.5
          ilatn =    1.
          ilonn =    1.
          nlat  =  180
          nlon  =  360
       elseif (pref=='GL_SM.GPNR.') then
          latni =  -89.875             !  trmm/navy + gpcp global
          latnf =   89.875
          lonni = -179.875
          lonnf =  179.875
          ilatn =    0.250
          ilonn =    0.250
          nlat  =  720
          nlon  = 1440
       elseif (pref=='GFS025gr.SOILM.') then
          latni =  -89.875             !  gfs
          latnf =   89.875
          lonni =  -179.875 ! 0.125
          lonnf =   179.875 !359.875
          ilatn =    0.250
          ilonn =    0.250
          nlat  =  721
          nlon  = 1441
       elseif(pref=='SM.GEOS.') then

		  !Opening the descriptor file to read info
		  print *, usdata(1:len_trim(usdata)-3)//'ctl'
		  open(2, status='old', form='formatted', action='read', &
			   file=usdata(1:len_trim(usdata)-3)//'ctl')

	      !First time: read variables and allocate all
		  read(2,*) lixo,dset
		  read(2,*) lixo,undef
		  read(2,*) lixo,title
		  read(2,*) lixo,nlon,lixo,lonni,ilonn
		  read(2,*) lixo,nlat,lixo,latni,ilatn
		  read(2,*) lixo,n4us
		  n4us=n4us-1
		  allocate(slz_us(0:n4us))
		  read(2,*) lixo,lixo,lixo,tdef
		  read(2,*) lixo,vars
		  allocate(SoilVarName(vars),Slevels(vars))


		  !Goto to begin of file and read all variables
		  rewind(unit=2)
		  read(2,*) lixo,dset
		  read(2,*) lixo,undef
		  read(2,*) lixo,title
		  read(2,*) lixo,nlon,lixo,lonni,ilonn
		  read(2,*) lixo,nlat,lixo,latni,ilatn
		  read(2,*) lixo,lixo,lixo,slz_us(0:n4us)
		  read(2,*) lixo,lixo,lixo,tdef
		  read(2,*) lixo,vars
		  do k=1,vars
		    read(2,*)  SoilVarName(k),Slevels(k)
		  enddo

		  close(unit=2)

		  !Determine the end of lats and lons
		  latnf=latni+(nlat-1)*ilatn
		  lonnf=lonni+(nlon-1)*ilonn

		  if(mchnum==master_num) then
			write(*,*) c_lightYellow
			write(*,*) '===================================='
			write(*,*) 'Input Soil Moisture File Inventory: '
			write(*,*) '===================================='
			write(*,*) 'Filename    : ',usdata(1:len_trim(usdata)-3)//'ctl'
			write(*,*) 'Bin filename: ',trim(dset)
			write(*,*) 'Undef Value : ',undef
			write(*,*) 'File Title  : ' ,trim(title)
			write(*,*) 'Lons        : ',nlon,lonni,lonnf,ilonn
			write(*,*) 'Lats        : ',nlat,latni,latnf,ilatn
			write(*,fmt='(A,I2.2,A)') ' # of levels : (00:',n4us,')'
			write(*,*) 'Levels      : ',slz_us(0:n4us)
			write(*,*) 'Time of data: ',trim(tdef)
			write(*,*) '# of vars   : ',vars
			do k=1,vars
				write(*,fmt='(A,I1,A,A,1X,I2.2)') ' Var(',k,')   : ',SoilVarName(k),Slevels(k)
			enddo
			write(*,*) '===================================='
			write(*,*) c_noColor

			!allocate(lats_(nlat),lons_(nlon))
			!do j=1,nlat
			!		lats_(i)=latni+(i-1)*ilatn
			!enddo
			!do i=1,nlon
			!		lons_(j)=lonni+(j-1)*ilonn
			!enddo

		  endif

		  allocate(us_geos(nlon,nlat,0:n4us))

	      if(mchnum==master_num) then
			!Reading the file
			open(2, status='old', form='unformatted', access='direct', &
					recl=4*nlat*nlon, file=usdata(1:len_trim(usdata)))
			nrec=1
			do k=0,n4us,1
        !Geos SM data is upside down
        ki=n4us-k
				read(2,rec=nrec) us_geos(1:nlon,1:nlat,ki)
				nrec=nrec+1
			enddo
			close(2)

			!open(unit=2,file='./us.gra',action='WRITE',status='REPLACE' &
			!	,form='UNFORMATTED',access='DIRECT', &
			!recl=4*nlat*nlon)
			!nrec=1
			!do k=0,n4us,1
			!	write (2,rec=nrec) us_geos(1:nlon,1:nlat,k)
			!	nrec=nrec+1
			!enddo
			!close(2)
			!
			!open(unit=2,file='./us.ctl',action='WRITE',status='replace' &
			!	,form='FORMATTED')
			!write(2,*) 'dset ^./us.gra'
			!write(2,*) 'undef',undef
			!write(2,*) 'title us'
			!write(2,*) 'xdef',nlon,'linear',lonni,ilonn
			!write(2,*) 'ydef',nlat,'linear',latni,ilatn
			!write(2,*) 'zdef',n4us+1,'levels',slz_us(0:n4us)
			!write(2,*) 'tdef   1 linear ',tdef,'  1hr'
			!write(2,*) 'vars 1'
			!write(2,*) 'us',n4us+1,'99 Us'
			!write(2,*) 'endvars'
			!close(2)
		endif

		call parf_bcast(us_geos, int(nlon,i8), int(nlat,i8), int(n4us+1,i8), &
						master_num)

       else
          call fatal_error('unexpected prefix for soil moisture')
       endif

       allocate(prlat(nlon,nlat), stat=ierr)
       if (ierr/=0) call fatal_error("error allocating prlats (soilmoistureinit)")
       allocate(prlon(nlon,nlat), stat=ierr)
       if (ierr/=0) call fatal_error("error allocating prlon (soilmoistureinit)")

       call apiprlatlon(nlon, nlat, prlat, prlon, ilatn, ilonn, latni, lonni)

       allocate(api_us(nlon,nlat,n4us), stat=ierr)
       if (ierr/=0) call fatal_error("error allocating api_us (soilmoistureinit)")


       allocate(api_temp(nlon,nlat,n4us), stat=ierr)
       if (ierr/=0) call fatal_error("error allocating api_temp (soilmoistureinit)")
       allocate(usdum(n4us), stat=ierr)
       if (ierr/=0) call fatal_error("error allocating usdum (soilmoistureinit)")

       if (mchnum==master_num) then
          print *, '-------------------------------------- grid=', ifm
          print *, 'opening soil moisture data= ', trim(usdata)
       endif

       if (.not.theref) then ! arquivo .gra
			if (pref=='us' .or. pref=='sm' .or. pref=='gfs025gr.soilm.' &
			    .or. pref=='SM.GEOS.') then   ! arquivo .gra acesso direto
				if (mchnum==master_num) then
					if (pref=='us' .or. pref=='sm')then
						open(2, status='old', form='unformatted', access='direct', &
							recl=4*nlat*nlon*n4us, file=usdata(1:len_trim(usdata)))
						read(unit=2,rec=1) api_us        ! water content
					elseif(pref=='gfs025gr.soilm.') then
						open(2, status='old', form='unformatted', access='direct', &
							recl=4*nlat*nlon, file=usdata(1:len_trim(usdata)))
						nrec=0
						do irec=1,n4us
							nrec=nrec+1
							read(unit=2,rec=nrec) api_us(:,:,n4us-irec+1)       ! water content
						enddo

						do irec=1,n4us
							nrec=nrec+1
							read(unit=2,rec=nrec) api_temp(:,:,n4us-irec+1)       ! water content
						enddo

						do k=1,n4us
							do j=1,nlat
								do i=1,nlon/2
									dummy=api_us(i,j,k)
									api_us(i,j,k)=api_us(i+nlon/2-1,j,k)
									api_us(i+nlon/2-1,j,k)=dummy
									!
									dummy=api_temp(i,j,k)
									api_temp(i,j,k)=api_temp(i+nlon/2-1,j,k)
									api_temp(i+nlon/2-1,j,k)=dummy
								enddo
							enddo
						enddo

				   !=====================================
                   !Nova versao que pega umidade do GEOS
                   !=====================================
				   elseif(pref=='SM.GEOS.') then
						do k=1,n4us,1
							api_us(1:nlon,1:nlat,k)=us_geos(1:nlon,1:nlat,k) ! wetness
						enddo
					endif
                   !======================================
				endif

				call parf_bcast(api_us, int(nlon,i8), int(nlat,i8), int(n4us,i8), &
						master_num)

				if(pref/='gfs025gr.soilm.') then
					call swap32(api_us, nlat*nlon*n4us) ! verify before call swap32 - demerval
					if ( (api_us(nlat/2,nlon/2,1)<0) .or. &
							(api_us(nlat/2,nlon/2,1)>1))     &
						call swap32(api_us, nlat*nlon*n4us)
					endif

!				else     ! arquivo .gra acesso sequencial
!					if (mchnum==master_num) then

!						open(2, status='old', form='unformatted', file=usdata(1:len_trim(usdata)))
!						do k=1,n4us
!							read(2) ((api_us(i,j,k),i=1,nlon), j=1,nlat) ! wetness
!						enddo
!						close(2)
!					endif

!					call parf_bcast(api_us, int(nlon,i8), int(nlat,i8), int(n4us,i8), &
!						master_num)
				endif

       else  ! arquivo .vfm
			if (mchnum==master_num) then
				open(unit=2, file=usdata(1:len_trim(usdata)), form='formatted', status='old')
				call vfirec(2, api_us, (nlat*nlon*n4us), 'lin')
				close(2)
			endif

			call parf_bcast(api_us, int(nlon,i8), int(nlat,i8), int(n4us,i8), &
               master_num)

       endif

       ! gathering data
       varn = 'soil_water'
       call gatherdata(idim_type, varn, ifm, mzg, nnxp(ifm), nnyp(ifm), &
            npat, nmachs, mchnum, mynum, master_num,                    &
            soil_water, globalsoilwater)
       varn = 'soil_text'
       call gatherdata(idim_type, varn, ifm, mzg, nnxp(ifm), nnyp(ifm), &
            npat, nmachs, mchnum, mynum, master_num,                    &
            soil_text, globalsoiltext)
       varn = 'glon'
       call gatherdata(2, varn, ifm, nnxp(ifm), nnyp(ifm), &
            nmachs, mchnum, mynum, master_num,             &
            glon, globalglon)
       varn = 'glat'
       call gatherdata(2, varn, ifm, nnxp(ifm), nnyp(ifm), &
            nmachs, mchnum, mynum, master_num,             &
            glat, globalglat)


       ! loop no dominio global do modelo
       do j=1,nnyp(ifm)
          do i=1,nnxp(ifm)
             ! evitando pontos fora do dominio da grade de umidade de solo
             if ( globalglat(i,j)<latni .or. globalglat(i,j)>latnf .or. &
                  globalglon(i,j)<lonni .or. globalglon(i,j)>lonnf) cycle

             call interpolacao(globalglon(i,j), globalglat(i,j), nlon, nlat, &
                  prlat, prlon, i1, i2, ic, j1, j2, jc)
!write(86,fmt='(6(I3.3,1X),4(F5.1,1X))') i,j,i1,j1,ic,jc,globalglon(i,j),prlon(ic,1),globalglat(i,j),prlat(1,jc)
             if (ic>=0 .and. jc>=0) then
                dlonr = 0.5*(globalglon(nnxp(ifm)-1,j) - globalglon(1,j))/&
                float(nnxp(ifm)-1)
                dlatr = 0.5*(globalglat(i,nnyp(ifm)-1) - globalglat(i,1))/&
                float(nnyp(ifm)-1)
!write(87,fmt='(2(I3.3,1X),2(F8.4,1X))') i,j,dlonr,dlatr
                qi1   = int(dlonr/ilonn+0.5)
                qi2   = int(dlonr/ilonn+0.5)
                qj1   = int(dlatr/ilatn+0.5)
                qj2   = int(dlatr/ilatn+0.5)

                do k=1,n4us
                   ncount = 0
                   usdum(k)=0.
                   do jj=max(1,jc-qj1),min(nlat,jc+qj2)
                      do ii=max(1,ic-qi1),min(nlon,ic+qi2)
                         if (api_us(ii,jj,k)>1.e-5) then
                            !If the value is undef probably is ocean
							if(pref=='SM.GEOS.') then
								if(api_us(ii,jj,k)>=undef) then
									api_us(ii,jj,k)=0.0
								endif
							endif
                            do ipat=2,npat
                               ncount = ncount + 1
                               usdum(k) = usdum(k) + api_us(ii,jj,k)
!write(88,fmt='(6(I4.4,1X),2(F5.1,1X),2(F18.6,1X))') i,j,ii,jj,k,ipat,prlon(ii,jj),prlat(ii,jj),api_us(ii,jj,k),usdum(k)
                            enddo
                         endif
                      enddo
                   enddo

                   usdum(k) = usdum(k)/(float(ncount) + 1.e-10)
                enddo

			         	!usdum_before(i,j,k)=usdum(k)

                do k=mzg,1,-1
                   do kk=n4us,1,-1
                      if (slz(k)>=slz_us(kk)) then
                         do ipat=2,npat
                            nsoil = nint(globalsoiltext(k,i,j,ipat))
							!- umidade em mm3/mm3 (gfs case)
							globalsoilwater(k,i,j,ipat) = usdum(kk+1)
!write(90,fmt='(6(I4.4,1X),3(F18.6,1X))') k,kk,i,j,ipat,nsoil,globalsoilwater(k,i,j,ipat),usdum(kk+1),slmsts(nsoil)
							! umidade lida em % (armazenamento)
                            if ( pref=='sm_v2.'      .or. &
                                 pref=='gl_sm.gpcp.' .or. &
                                 pref=='gl_sm.gpnr.' .or. &
                                 pref=='SM.GEOS.') then
								globalsoilwater(k,i,j,ipat) = &
                                    globalsoilwater(k,i,j,ipat)*slmsts(nsoil)
                            endif
                            if (usdum(kk+1)<1.e-5) &
								globalsoilwater(k,i,j,ipat) = slmstr(k)*slmsts(nsoil) !oceano
							globalsoilwater(k,i,j,ipat) = max(soilcp(nsoil), &
                                                              min(globalsoilwater(k,i,j,ipat), &
                                                               slmsts(nsoil)))
!write(91,fmt='(6(I4.4,1X),3(F18.6,1X))') k,kk,i,j,ipat,nsoil,globalsoilwater(k,i,j,ipat),usdum(kk+1),slmsts(nsoil)
                         enddo
                         exit
                      elseif (slz(k)<slz_us(1)) then
                         do ipat=2,npat
                            nsoil = nint(globalsoiltext(k,i,j,ipat))
							!- umidade em mm3/mm3 (gfs case)
                            globalsoilwater(k,i,j,ipat) = usdum(1)
							! umidade lida em % (armazenamento) - v2 (sm_v2.)
                            if ( pref=='sm_v2.'      .or. &
                                 pref=='gl_sm.gpcp.' .or. &
                                 pref=='gl_sm.gpnr.' .or. &
                                 pref=='SM.GEOS.') then
                                 globalsoilwater(k,i,j,ipat) = &
                                    globalsoilwater(k,i,j,ipat)*slmsts(nsoil)
                            endif
                            if (usdum(1)<1.e-5)   &
                                 globalsoilwater(k,i,j,ipat) = slmstr(k)*slmsts(nsoil)!oceano
                            globalsoilwater(k,i,j,ipat) = max(soilcp(nsoil), &
                                                               min(globalsoilwater(k,i,j,ipat), &
                                                               slmsts(nsoil)))
!write(92,fmt='(6(I4.4,1X),3(F18.6,1X))') k,kk,i,j,ipat,nsoil,globalsoilwater(k,i,j,ipat),usdum(kk+1),slmsts(nsoil)

                         enddo
                         exit
                      endif
                   enddo
                enddo
             endif
          enddo
       enddo



!    open(unit=2,file='./globalsoilwater.gra',&
!        action='WRITE',status='REPLACE',form='UNFORMATTED',access='DIRECT', &
!        recl=4*80*80)
!    irec=1
!    do k=1,mzg
!		write (2,rec=irec) globalsoilwater(k,1:80,1:80,2)
!		irec=irec+1
!     enddo
!    do k=1,mzg
!		write (2,rec=irec) globalsoilwater(k,1:80,1:80,3)
!		irec=irec+1
!    enddo
!    do k=1,mzg
!		write (2,rec=irec) globalsoilwater(k,1:80,1:80,4)
!		irec=irec+1
!    enddo
!    close(2)

!    open(unit=2,file='./globalsoilwater.ctl' &
!         ,action='WRITE',status='replace',form='FORMATTED')

!    write(2,*) 'dset ^./globalsoilwater.gra'
!    write(2,*) 'undef -0.9990000E+34'
!    write(2,*) 'title Soil Water'
!    write(2,*) 'xdef 80 linear -48.6439743  0.0964959'
!    write(2,*) 'ydef 80 linear -24.6234856  0.0899214'
!    write(2,*) 'zdef ',mzg,' levels ',(i,i=1,mzg)
!    write(2,*) 'tdef   1 linear 00:00z06may2020           1hr'
!    write(2,*) 'vars 3'
!    write(2,*) 'sw2 ',mzg,'99 Usdum'
!    write(2,*) 'sw3 ',mzg,'99 Usdum'
!    write(2,*) 'sw4 ',mzg,'99 Usdum'
!    write(2,*) 'endvars'

!    close(2)

     deallocate(api_us, usdum, prlat, prlon,api_temp)


     if (mchnum==master_num) then
          open(2, status='unknown', form='unformatted', access='direct', &
               recl=4*nnxp(ifm)*nnyp(ifm)*mzg*npat, file=usmodel(1:len_trim(usmodel)))
          write(unit=2,rec=1) globalsoilwater
          close(2)
     endif

     ! scattering local data
     call mk_4_buff(globalsoilwater(:,:,:,:), soil_water(:,:,:,:), &
            mzg, nnxp(ifm), nnyp(ifm), npat, mzg, n2, n3, npat, ia, &
            iz, ja, jz)

!    open(unit=2,file='./soilwater.gra',&
!        action='WRITE',status='REPLACE',form='UNFORMATTED',access='DIRECT', &
!        recl=4*80*80)
!    irec=1
!    do k=1,mzg
!		write (2,rec=irec) soil_water(k,1:80,1:80,2)
!		irec=irec+1
!     enddo
!    do k=1,mzg
!		write (2,rec=irec) soil_water(k,1:80,1:80,3)
!		irec=irec+1
!    enddo
!    do k=1,mzg
!		write (2,rec=irec) soil_water(k,1:80,1:80,4)
!		irec=irec+1
!    enddo
!    close(2)
!
!    open(unit=2,file='./soilwater.ctl' &
!         ,action='WRITE',status='replace',form='FORMATTED')
!
!    write(2,*) 'dset ^./soilwater.gra'
!    write(2,*) 'undef -0.9990000E+34'
!    write(2,*) 'title Soil Water'
!    write(2,*) 'xdef 80 linear -48.6439743  0.0964959'
!    write(2,*) 'ydef 80 linear -24.6234856  0.0899214'
!    write(2,*) 'zdef ',mzg,' levels ',(i,i=1,mzg)
!    write(2,*) 'tdef   1 linear 00:00z06may2020           1hr'
!    write(2,*) 'vars 3'
!    write(2,*) 'sw2 ',mzg,'99 Usdum'
!    write(2,*) 'sw3 ',mzg,'99 Usdum'
!    write(2,*) 'sw4 ',mzg,'99 Usdum'
!    write(2,*) 'endvars'
!
!    close(2)

!- 8/9/2015 srf
!    because the the soil moisture dataset interpolated to the model grid
!    is never checked again, we will allways require the interpolation to
!    be done every model initialization. this will prevents soil moisture
!    interpolated for given grid specification being erroneously be used
!    for a different model configuration
!
!    else
!       if (mchnum==master_num) then
!          print*,'-------------------------------------- grid=', ifm
!          print*,'opening soil moisture file= ', trim(usmodel)
!          open(2, status='old', form='unformatted', access='direct', &
!               recl=4*nnxp(ifm)*nnyp(ifm)*mzg*npat, file=usmodel(1:len_trim(usmodel)))
!          read(unit=2,rec=1) globalsoilwater
!          close(2)
!          print*,'--------------------------------------'
!       endif
!
!       call parf_bcast(globalsoilwater, int(mzg,i8), int(nnxp(ifm),i8), &
!            int(nnyp(ifm),i8), int(npat,i8), master_num)
!
!       ! distributing local information about soil water
!       call mk_4_buff(globalsoilwater(:,:,:,:), soil_water(:,:,:,:), &
!            mzg, nnxp(ifm), nnyp(ifm), npat, mzg, n2, n3, npat, ia, iz, ja, jz)
!
!    endif
!-srf end


    !----- recalculate soil_energy
    c1 = 0.5*cpi

    do j=1,n3
       do i=1,n2

          k2      = lpw(i,j)
          pis     = c1*(pi0(k2-1,i,j) + pi0(k2,i,j) + pp(k2-1,i,j) + pp(k2,i,j))
          airtemp = theta(k2,i,j)*pis

          do ipat=2,npat
             do k=1,mzg
               nsoil = nint(soil_text(k,i,j,ipat))
                soil_energy(k,i,j,ipat) = (airtemp - 273.15 + stgoff(k))*   &
                     (slcpd(nsoil) + soil_water(k,i,j,ipat)*4.186e6)      + &
                     soil_water(k,i,j,ipat)*3.34e8
             enddo
          enddo
       enddo
    enddo

!    print *,'LFR:',n2,n3,npat,mzg,nsoil

!     open(unit=2,file='./moisture.gra',&
!        action='WRITE',status='REPLACE',form='UNFORMATTED',access='DIRECT', &
!        recl=4*80*80)
!    irec=1
!    do k=1,mzg
!		write (2,rec=irec) soil_water(k,1:80,1:80,2)
!		irec=irec+1
!     enddo
!    do k=1,mzg
!		write (2,rec=irec) soil_water(k,1:80,1:80,3)
!		irec=irec+1
!     enddo
!    do k=1,mzg
!		write (2,rec=irec) soil_water(k,1:80,1:80,4)
!		irec=irec+1
!    enddo
!    do k=1,mzg
!		write (2,rec=irec) soil_energy(k,1:80,1:80,2)
!		irec=irec+1
!    enddo
!    do k=1,mzg
!		write (2,rec=irec) soil_energy(k,1:80,1:80,3)
!		irec=irec+1
!    enddo
!    do k=1,mzg
!		write (2,rec=irec) soil_energy(k,1:80,1:80,4)
!		irec=irec+1
!    enddo


!        close(2)

!    open(unit=2,file='./moisture.ctl' &
!         ,action='WRITE',status='replace',form='FORMATTED')

!    write(2,*) 'dset ^./moisture.gra'
!    write(2,*) 'undef -0.9990000E+34'
!    write(2,*) 'title Soil Water'
!    write(2,*) 'xdef 80 linear -48.6439743  0.0964959'
!    write(2,*) 'ydef 80 linear -24.6234856  0.0899214'
!    write(2,*) 'zdef ',mzg,' levels ',(i,i=1,mzg)
!    write(2,*) 'tdef   1 linear 00:00z06may2020           1hr'
!    write(2,*) 'vars 6'
!    write(2,*) 'sw2 ',mzg,'99 Usdum'
!    write(2,*) 'sw3 ',mzg,'99 Usdum'
!    write(2,*) 'sw4 ',mzg,'99 Usdum'
!    write(2,*) 'se2 ',mzg,'99 Usdum'
!    write(2,*) 'se3 ',mzg,'99 Usdum'
!    write(2,*) 'se4 ',mzg,'99 Usdum'
!    write(2,*) 'endvars'

!    close(2)



  end subroutine soilMoistureInit


!!$  ! Recreating Global Information (Gathering data)
!!$  SUBROUTINE gatherData2D(idim_type, varn, ifm, mzg, nnxp, nnyp, npat, nwave, &
!!$       nmachs, mchnum, mynum, master_num,                                   &
!!$       localData2D, globalData2D)
!!$
!!$    USE ReadBcst, ONLY: &
!!$         PreProcAndGather, & ! Subroutine
!!$         LocalSizesAndDisp   ! Subroutine
!!$    USE mem_grid, ONLY : &
!!$         GlobalSizes         ! Subroutine
!!$    USE ParLib, ONLY: &
!!$         parf_bcast ! Subroutine
!!$
!!$    IMPLICIT NONE
!!$    INCLUDE "i8.h"
!!$    ! Arguments:
!!$    INTEGER, INTENT(IN)           :: idim_type, ifm, mzg, nnxp, nnyp, npat, &
!!$         nwave, nmachs, mchnum, mynum, master_num
!!$    CHARACTER(LEN=16), INTENT(IN) :: varn
!!$    REAL, INTENT(IN)              :: localData2D(:,:)
!!$    REAL, INTENT(OUT)             :: globalData2D(:,:)
!!$    ! Local Variables:
!!$    CHARACTER(LEN=16)  :: localVarn
!!$    INTEGER            :: ierr
!!$    INTEGER, PARAMETER :: idim_type_min = 2
!!$    INTEGER, PARAMETER :: idim_type_max = 7
!!$    INTEGER            :: il1(nmachs)
!!$    INTEGER            :: ir2(nmachs)
!!$    INTEGER            :: jb1(nmachs)
!!$    INTEGER            :: jt2(nmachs)
!!$    INTEGER            :: localSize(nmachs,idim_type_min:idim_type_max)
!!$    INTEGER            :: disp(nmachs,idim_type_min:idim_type_max)
!!$    INTEGER            :: maxLocalSize
!!$    INTEGER            :: sizeGathered(idim_type_min:idim_type_max)
!!$    INTEGER            :: maxSizeGathered
!!$    INTEGER            :: sizeFullField(idim_type_min:idim_type_max)
!!$    INTEGER            :: maxsizeFullField
!!$    INTEGER            :: globalSize(idim_type_min:idim_type_max)
!!$    REAL, ALLOCATABLE  :: localChunk(:)
!!$    REAL, ALLOCATABLE  :: gathered(:)
!!$    REAL, ALLOCATABLE  :: fullField(:)
!!$
!!$    ! Recreating Global information about Soil Water
!!$    ! grid dependent, field independent constants for gather and unpacking
!!$    ! as a function of idim_type
!!$    CALL LocalSizesAndDisp(ifm, il1, ir2, jb1, jt2, localSize, disp)
!!$    maxLocalSize = MAXVAL(localSize(mynum,:))
!!$    ALLOCATE(localChunk(maxLocalSize), stat=ierr)
!!$    IF (ierr/=0) THEN
!!$       CALL fatal_error("Error allocating localChunk (gatherData)")
!!$    ENDIF
!!$    CALL CopyLocalChunk(localData2D(1,1), localChunk, &
!!$         LocalSize(mynum,idim_type))
!!$    sizeGathered(:) = disp(nmachs,:) + localSize(nmachs,:)
!!$    maxSizeGathered = MAXVAL(sizeGathered)
!!$    ALLOCATE(gathered(maxSizeGathered), stat=ierr)
!!$    IF (ierr/=0) THEN
!!$       CALL fatal_error("Error allocating gathered (gatherData)")
!!$    ENDIF
!!$    ! grid dependent field sizes as a function of idim_type
!!$    CALL GlobalSizes(ifm, nmachs, nwave, globalSize)
!!$    IF (mchnum==master_num) THEN
!!$       sizeFullField(:) = globalSize(:)
!!$    ELSE
!!$       sizeFullField(:) = 1
!!$    END IF
!!$    maxSizeFullField = MAXVAL(sizeFullField)
!!$    ALLOCATE(fullField(sizeFullField(idim_type)), stat=ierr)
!!$    IF (ierr/=0) THEN
!!$       CALL fatal_error("Error allocating fullField (gatherData)")
!!$    ENDIF
!!$    localVarn = trim(varn)
!!$    CALL PreProcAndGather(.FALSE., ifm, idim_type, localVarn, &
!!$         il1, ir2, jb1, jt2, localSize, disp,                 &
!!$         localSize(mynum,idim_type), LocalChunk,              &
!!$         sizeGathered(idim_type), gathered,                   &
!!$         sizeFullField(idim_type), fullField                  )
!!$
!!$    IF (mchnum==master_num) THEN
!!$       globalData2D = RESHAPE(fullField, (/nnxp, nnyp/))
!!$    ENDIF
!!$
!!$    call parf_bcast(globalData2D, int(nnxp,i8), int(nnyp,i8), master_num)
!!$
!!$    DEALLOCATE(fullField)
!!$    DEALLOCATE(gathered)
!!$    DEALLOCATE(localChunk)
!!$
!!$  END SUBROUTINE gatherData2D
!!$
!!$  ! Recreating Global Information (Gathering data)
!!$  SUBROUTINE gatherData4D(idim_type, varn, ifm, mzg, nnxp, nnyp, npat, nwave, &
!!$       nmachs, mchnum, mynum, master_num,                                   &
!!$       localData4D, globalData4D)
!!$
!!$    USE ReadBcst, ONLY: &
!!$         PreProcAndGather, & ! Subroutine
!!$         LocalSizesAndDisp   ! Subroutine
!!$    USE mem_grid, ONLY : &
!!$         GlobalSizes         ! Subroutine
!!$    USE ParLib, ONLY: &
!!$         parf_bcast ! Subroutine
!!$
!!$    IMPLICIT NONE
!!$    INCLUDE "i8.h"
!!$    ! Arguments:
!!$    INTEGER, INTENT(IN)           :: idim_type, ifm, mzg, nnxp, nnyp, npat, &
!!$         nwave, nmachs, mchnum, mynum, master_num
!!$    CHARACTER(LEN=16), INTENT(IN) :: varn
!!$    REAL, INTENT(IN)              :: localData4D(:,:,:,:)
!!$    REAL, INTENT(OUT)             :: globalData4D(:,:,:,:)
!!$    ! Local Variables:
!!$    CHARACTER(LEN=16)  :: localVarn
!!$    INTEGER            :: ierr
!!$    INTEGER, PARAMETER :: idim_type_min = 2
!!$    INTEGER, PARAMETER :: idim_type_max = 7
!!$    INTEGER            :: il1(nmachs)
!!$    INTEGER            :: ir2(nmachs)
!!$    INTEGER            :: jb1(nmachs)
!!$    INTEGER            :: jt2(nmachs)
!!$    INTEGER            :: localSize(nmachs,idim_type_min:idim_type_max)
!!$    INTEGER            :: disp(nmachs,idim_type_min:idim_type_max)
!!$    INTEGER            :: maxLocalSize
!!$    INTEGER            :: sizeGathered(idim_type_min:idim_type_max)
!!$    INTEGER            :: maxSizeGathered
!!$    INTEGER            :: sizeFullField(idim_type_min:idim_type_max)
!!$    INTEGER            :: maxsizeFullField
!!$    INTEGER            :: globalSize(idim_type_min:idim_type_max)
!!$    REAL, ALLOCATABLE  :: localChunk(:)
!!$    REAL, ALLOCATABLE  :: gathered(:)
!!$    REAL, ALLOCATABLE  :: fullField(:)
!!$
!!$    ! Recreating Global information about Soil Water
!!$    ! grid dependent, field independent constants for gather and unpacking
!!$    ! as a function of idim_type
!!$    CALL LocalSizesAndDisp(ifm, il1, ir2, jb1, jt2, localSize, disp)
!!$    maxLocalSize = MAXVAL(localSize(mynum,:))
!!$    ALLOCATE(localChunk(maxLocalSize), stat=ierr)
!!$    IF (ierr/=0) THEN
!!$       CALL fatal_error("Error allocating localChunk (gatherData)")
!!$    ENDIF
!!$    CALL CopyLocalChunk(localData4D(1,1,1,1), localChunk, &
!!$         LocalSize(mynum,idim_type))
!!$    sizeGathered(:) = disp(nmachs,:) + localSize(nmachs,:)
!!$    maxSizeGathered = MAXVAL(sizeGathered)
!!$    ALLOCATE(gathered(maxSizeGathered), stat=ierr)
!!$    IF (ierr/=0) THEN
!!$       CALL fatal_error("Error allocating gathered (gatherData)")
!!$    ENDIF
!!$    ! grid dependent field sizes as a function of idim_type
!!$    CALL GlobalSizes(ifm, nmachs, nwave, globalSize)
!!$    IF (mchnum==master_num) THEN
!!$       sizeFullField(:) = globalSize(:)
!!$    ELSE
!!$       sizeFullField(:) = 1
!!$    END IF
!!$    maxSizeFullField = MAXVAL(sizeFullField)
!!$    ALLOCATE(fullField(sizeFullField(idim_type)), stat=ierr)
!!$    IF (ierr/=0) THEN
!!$       CALL fatal_error("Error allocating fullField (gatherData)")
!!$    ENDIF
!!$    localVarn = trim(varn)
!!$    CALL PreProcAndGather(.FALSE., ifm, idim_type, localVarn, &
!!$         il1, ir2, jb1, jt2, localSize, disp,                 &
!!$         localSize(mynum,idim_type), LocalChunk,              &
!!$         sizeGathered(idim_type), gathered,                   &
!!$         sizeFullField(idim_type), fullField                  )
!!$
!!$    IF (mchnum==master_num) THEN
!!$       globalData4D = RESHAPE(fullField, (/mzg, nnxp, nnyp, npat/))
!!$    ENDIF
!!$
!!$    call parf_bcast(globalData4D, &
!!$         int(mzg,i8), int(nnxp,i8), int(nnyp,i8), int(npat,i8), master_num)
!!$
!!$    DEALLOCATE(fullField)
!!$    DEALLOCATE(gathered)
!!$    DEALLOCATE(localChunk)
!!$
!!$  END SUBROUTINE gatherData4D
  subroutine StoreNamelistFileAtSoilMoisture(oneNamelistFile)
    implicit none
    type(namelistFile), pointer :: oneNamelistFile
    soil_moist = oneNamelistFile%soil_moist
    soil_moist_fail = oneNamelistFile%soil_moist_fail
    usdata_in = oneNamelistFile%usdata_in
    usmodel_in = oneNamelistFile%usmodel_in
  end subroutine StoreNamelistFileAtSoilMoisture
END MODULE soilMoisture

  !
  ! prlatlon
  !----------------------------------------------------------------
  ! SUB-ROTINA QUE ESTABELECE LATITUDES E LONGITUDES DOS PONTOS DE
  ! GRADE DO CAMPO DE PRECIPITACAO
  SUBROUTINE apiPrlatlon(nlon, nlat, prlat, prlon, ilatn, ilonn, latni, lonni)
    IMPLICIT NONE
    ! Arguments:
    INTEGER, INTENT(IN) :: nlon, nlat
    REAL, INTENT(OUT)   :: prlat(nlon,nlat) !(nlon,nlat)
    REAL, INTENT(OUT)   :: prlon(nlon,nlat) !(nlon,nlat)
    REAL, INTENT(IN)    ::  ilatn, ilonn, latni, lonni
    ! Local Variables:
    INTEGER :: i, j

    DO j=1,nlat
       DO i=1,nlon
          prlon(i,j) = lonni + (i-1)*ilonn
          prlat(i,j) = latni + (j-1)*ilatn
       ENDDO
    ENDDO

  END SUBROUTINE apiPrlatlon

  !----------------------------------------------------------------
  ! interpolacao
  !----------------------------------------------------------------
  ! SUB-ROTINA QUE REALIZA INTERPOLACAO ENTRE GRADES (RAMS E UMIDADE DO SOLO)
  SUBROUTINE interpolacao(glon, glat, nlon, nlat, prlat, prlon, &
       i1, i2, ic, j1, j2, jc)

    IMPLICIT NONE
    ! Arguments:
    INTEGER, INTENT(OUT) :: i1, i2, ic, j1, j2, jc
    REAL, INTENT(IN)     :: glat, glon
    INTEGER, INTENT(IN)  :: nlon, nlat
    REAL, INTENT(IN)     :: prlat(nlon,nlat)
    REAL, INTENT(IN)     :: prlon(nlon,nlat)
    ! Local Variables
    REAL    :: diffx1, diffx2, diffy1, diffy2
    INTEGER :: i, j

    DO i=1,nlon
       IF (glon<=prlon(i,1)) EXIT
    ENDDO
    i2 = i
    i1 = i-1

    DO j=1,nlat
       IF (glat<=prlat(1,j)) EXIT
    ENDDO
    j2 = j
    j1 = j-1

    diffx1 =   glon - prlon(i1,j1)
    diffx2 = -(glon - prlon(i1,j2))
    diffy1 =   glat - prlat(i1,j1)
    diffy2=  -(glat - prlat(i2,j1))

    jc = j1
    ic = i1
    IF (diffx1>diffx2) ic = i2
    IF (diffy1>diffy2) jc = j2

    IF (i1<1 .OR. i1>nlon .OR. j1<1 .OR. j1>nlat) THEN
       ic = -9999
       jc = -9999
    ENDIF

  END SUBROUTINE interpolacao

  !----------------------------------------------------------------

  SUBROUTINE changeDay(idate1, imonth1, iyear1, INT_DIF_TIME, &
       idate2, imonth2, iyear2)
    IMPLICIT NONE
    ! Arguments:
    INTEGER, INTENT(IN)  :: idate1, imonth1, iyear1, INT_DIF_TIME
    INTEGER, INTENT(OUT) :: idate2, imonth2, iyear2
    ! Local Variables:
    INTEGER :: i, increm, DMES(12)

    ! Initiate DMES
    DMES = (/31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31/)

    iyear2  = iyear1
    imonth2 = imonth1
    idate2  = idate1

    increm  = 1

    IF (INT_DIF_TIME<1) increm = increm*(-1)

    DO i=1,ABS(INT_DIF_TIME)
       idate2 = idate2 + increm
       IF (idate2<1) THEN
          imonth2 = imonth2 + increm
          IF (imonth2<1) THEN
             imonth2 = 12
             iyear2  = iyear2 - 1
          ENDIF
          idate2 = DMES(imonth2)
       ELSEIF (idate2>DMES(imonth2)) THEN
          imonth2 = imonth2 + increm
          IF (imonth2>12) THEN
             imonth2 = 1
             iyear2  = iyear2 + 1
          ENDIF
          idate2 = 1
       ENDIF
    ENDDO

  END SUBROUTINE changeDay


!-srf END MODULE soilMoisture

!============================================================================

SUBROUTINE Swap32(A, N)
  !
  !      REVERSE ORDER OF BYTES IN INTEGER*4 WORD, or REAL*4
  !
  IMPLICIT NONE
  ! Arguments:
  INTEGER, INTENT(IN)            :: n
  INTEGER(kind=4), INTENT(INOUT) :: a(n)
  ! Local Varaibles:
  CHARACTER(LEN=1) :: jtemp(4)
  CHARACTER(LEN=1) :: ktemp
  !
  ! Local variables
  INTEGER :: i, itemp

  EQUIVALENCE (jtemp(1), itemp)
  !
  SAVE
  !
  DO i=1,n
     itemp    = a(i)
     ktemp    = jtemp(4)
     jtemp(4) = jtemp(1)
     jtemp(1) = ktemp
     ktemp    = jtemp(3)
     jtemp(3) = jtemp(2)
     jtemp(2) = ktemp
     a(i)     = itemp
  ENDDO

END SUBROUTINE Swap32
